<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * Rooms View
 *
 * @since  1.0.0
 */
class JRoomBookingViewRooms extends JViewLegacy
{
    /**
     * Display the Rooms view
     *
     * @param   string  $tpl  The name of the template file to parse; automatically searches through the template paths.
     *
     * @return  void
     * @since 1.0.0
     */
    function display($tpl = null)
    {
        // Get application
        $app = JFactory::getApplication();
        $context = "jroombooking.list.admin.jroombooking";

        // Get data from the model
        $this->items		= $this->get('Items');
        $this->pagination	= $this->get('Pagination');
        $this->state			= $this->get('State');
        $this->filter_order 	= $app->getUserStateFromRequest($context.'filter_order', 'filter_order', 'id', 'cmd');
        $this->filter_order_Dir = $app->getUserStateFromRequest($context.'filter_order_Dir', 'filter_order_Dir', 'asc', 'cmd');
        $this->filterForm    	= $this->get('FilterForm');
        $this->activeFilters 	= $this->get('ActiveFilters');

        // What Access Permissions does this user have? What can (s)he do?
        $this->canDo = JRoomBookingHelper::getActions();

        // Check for errors.
        if (count($errors = $this->get('Errors')))
        {
            JFactory::getApplication()->enqueueMessage(implode('<br />', $errors), 'error');

            return false;
        }

        // Set the submenu
        JRoomBookingHelper::addSubmenu('rooms');
        $this->sidebar = JHtmlSidebar::render();

        // Set the toolbar and number of found items
        $this->addToolBar();

        // Display the template
        parent::display($tpl);

        // Set the document
        $this->setDocument();
    }

    /**
     * Add the page title and toolbar.
     *
     * @return  void
     *
     * @since   1.0.0
     */
    protected function addToolBar()
    {
        require_once JPATH_COMPONENT . '/helpers/jroombooking.php';
        $title = JText::_('COM_JROOMBOOKING_MANAGER_ROOMS');

        if ($this->pagination->total)
        {
            $title .= "<span style='font-size: 0.5em; vertical-align: middle;'>(" . $this->pagination->total . ")</span>";
        }

        JToolBarHelper::title($title, 'jroombooking');

        if ($this->canDo->get('core.create'))
        {
            JToolBarHelper::addNew('jroombooking.add', 'JTOOLBAR_NEW');
        }
        if ($this->canDo->get('core.edit'))
        {
            JToolBarHelper::editList('jroombooking.edit', 'JTOOLBAR_EDIT');
        }
        if ($this->canDo->get('core.delete'))
        {
            JToolBarHelper::deleteList('', 'rooms.delete', 'JTOOLBAR_DELETE');
        }
        if ($this->canDo->get('core.admin'))
        {
            JToolBarHelper::divider();
            JToolBarHelper::preferences('com_jroombooking');
        }
    }
    /**
     * Method to set up the document properties
     *
     * @return void
     * @since 1.0.0
     */
    protected function setDocument()
    {
        $document = JFactory::getDocument();
        $document->setTitle(JText::_('COM_JROOMBOOKING_ADMINISTRATION'));
    }
}