<?php
/**
 * @package     Room reservations
 * @subpackage  Room reservations component
 *
 * @author      Miloslav Schmied
 * @copyright   Copyright (C) 2016 - Ergotep, družstvo invalidů
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

// No direct access to this file
defined('_JEXEC') or die('Restricted access');

/**
 * ReserveRequest Model
 *
 * @since  1.0.0
 */
class JRoomBookingModelReserveRequest extends JModelForm
{

    protected $data;
    protected $room;

    /**
     * Metoda pro ziskani msitnosti z DB.
     *
     * @return mixed
     * @since 1.0.0
     */
    public function getRoom()
    {
        if (!isset($this->room)) {
            $jinput = JFactory::getApplication()->input;
            $id = $jinput->get('room_id', 0, 'INT');
            if ($id != 0) {
                $db = JFactory::getDbo();
                $query = $db->getQuery(true);
                $query->select($db->qn(array('id', 'name', 'description', 'photo', 'max_capacity', 'published')))
                    ->from($db->qn('#__jroombooking_rooms'))
                    ->where($db->qn('id') . ' = ' . $id);
                $db->setQuery($query);
                $this->room = $db->loadObject();
            }
        }

        return $this->room;
    }

    /**
     * Method to get the record form.
     *
     * @param   array    $data      Data for the form.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  mixed    A JForm object on success, false on failure
     *
     * @since   1.0.0
     */
    public function getForm($data = array(), $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm(
            'com_jroombooking.reserverequest',
            'reserverequest',
            array(
                'control' => 'jform',
                'load_data' => $loadData
            )
        );

        if (empty($form))
        {
            return false;
        }

        return $form;
    }

    /**
     * Method to get the registration form data.
     *
     * The base form data is loaded and then an event is fired
     * for users plugins to extend the data.
     *
     * @return  mixed  Data object on success, false on failure.
     *
     * @since   1.0.0
     */
    public function getData()
    {
        if ($this->data === null)
        {
            $this->data = new stdClass;
            $app = JFactory::getApplication();
            $params = JComponentHelper::getParams('com_jroombooking');

            // Override the base user data with any data in the session.
            $temp = (array) $app->getUserState('com_jroombooking.reserverequest.data', array());

            foreach ($temp as $k => $v)
            {
                $this->data->$k = $v;
            }
        }

        return $this->data;
    }

    /**
     * Method to get the data that should be injected in the form.
     *
     * @return  mixed  The data for the form.
     *
     * @since   1.0.0
     */
    protected function loadFormData()
    {
        $data = $this->getData();

        return $data;
    }

    /**
     * Method to auto-populate the model state.
     *
     * Note. Calling getState in this method will result in recursion.
     *
     * @return  void
     *
     * @since   1.0.0
     */
    protected function populateState()
    {
        // Get the application object.
        $app = JFactory::getApplication();
        $params = $app->getParams('com_jroombooking');

        // Load the parameters.
        $this->setState('params', $params);
    }

    /**
     * Method to get the script that have to be included on the form
     *
     * @return string	Script files
     * @since 1.0.0
     */
    public function getScript()
    {
        return 'components/com_jroombooking/models/forms/reserverequest.js';
    }

    /**
     * Metoda pro validaci terminu rezervace. Termin nesmi kolidovat s jiz schvalenymi rezervacemi.
     *
     * @param string $startDate Datum zacatku rezervace
     * @param string $endDate Datum konce rezervace
     * @return bool Vysledek validace
     * @since 1.0.0
     */
    public function validateDates($startDate, $endDate) {
        $valid = false;
        $jinput = JFactory::getApplication()->input;
        $id = $jinput->get('room_id', 0, 'INT');
        if ($id != 0) {
            $db = JFactory::getDbo();
            $query = $db->getQuery(true);
            $query->select('COUNT(' . $db->qn('id') . ')')
                ->from($db->qn('#__jroombooking_reservations'))
                ->where($db->qn('approved') . ' = ' . $db->q(1) . ' AND ' . $db->qn('room_id') . ' = ' . $db->q($id)
                    . ' AND ((' . $db->qn('start_date') . ' >= ' . $db->q($startDate)
                    . ' AND ' . $db->qn('end_date') . ' <= ' . $db->q($endDate) . ') OR (' . $db->qn('end_date') . ' >= ' . $db->q($startDate)
                    . ' AND ' . $db->qn('start_date') . ' <= '. $db->q($endDate) .'))');
            $db->setQuery($query);
            $result = $db->loadResult();
            if ($result == 0) $valid = true;
        }
        return $valid;
    }

    /**
     * Validuje data z prvniho formulare zadosti o rezervaci.
     *
     * @param array $data Data z formulare
     * @return array|bool Pokud je validace v poradku, vrati true, jinak vrati pole s chybami
     * @since 1.0.0
     */
    public function validateRequest($data) {
        $errors = array();

        $startDate = new JDate($data['start_date']);
        $endDate = new JDate($data['end_date']);
        //zacatek rezervace  = dnesek + X dni; X = 1
        $earliest = 1;
        $signum = $earliest >= 0 ? '+' : '-';
        $earliestDate = new JDate('NOW ' . $signum . ' ' . abs($earliest) . ' DAY');

        //kontrola, jestli neni datum zacatku rezervace starsi nez dnesek
        if ($startDate->format('Y-m-d') < $earliestDate->format('Y-m-d')) {
            $errors[] = JText::sprintf(
              'COM_JROOMBOOKING_RESERVE_START_DATE_LESS_OR_EQUAL_NOW', $earliestDate->format(JText::_('COM_JROOMBOOKING_DATE_FORMAT'), false, true)
            );
        }

        //datum zacatku musi byt drive nez datum konce
        if ($startDate >= $endDate) {
            $errors[] = JText::_('COM_JROOMBOOKING_RESERVE_START_DATE_LESS_OR_EQUAL_END_DATE');
        }

        //kontrola, jestli neni pocet mist pozadavku vetsi, nez max. kapacita pokoje
        if ($data['capacity'] > $this->getRoom()->max_capacity) {
            $errors[] = JText::_('COM_JROOMBOOKING_RESERVE_CAPACITY_EXCEEDED');
        }

        //kontrola, jestli datum rezervace pokoje nekoliduje s jiz schvalenou rezervaci
        if (!$this->validateDates($startDate->format('Y-m-d'), $endDate->format('Y-m-d'))) {
            $errors[] = JText::_('COM_JROOMBOOKING_RESERVE_ROOM_OCCUPIED');
            //ziskani alternativnich pokoju, ktere nejsou obsazene pro zadany termin a kapacitu
            $rooms = $this->getAlternativeRooms($data['capacity'], $startDate->toSql(), $endDate->toSql());
            if ($rooms !== false) {
                //vypsani alternativnich pokoju
                $errors[] = JText::_('COM_JROOMBOOKING_RESERVE_ROOM_OCCUPIED_ALTERNATIVES');
                foreach ($rooms as $room) {
                    $errors[] = '<a href="' . JRoute::_('index.php?option=com_jroombooking&view=reserverequest&room_id=' . $room->id, false) . '">' .
                                $room->name . '</a>';
                }
            } else {
                //zadny alternativni pokoj neexistuje
                $errors[] = JText::_('COM_JROOMBOOKING_RESERVE_ROOM_OCCUPIED_NO_ALTERNATIVE');
            }
        }

        return empty($errors) ? true : $errors;
    }

    private function getAlternativeRooms($capacity, $startDate, $endDate) {
        $db = JFactory::getDbo();
        $subQuery = $db->getQuery(true);

        $subQuery->select($db->qn('room_id'))
            ->from($db->qn('#__jroombooking_reservations'))
            ->where($db->qn('approved') . ' = ' . $db->q(1)
                . ' AND ((' . $db->qn('start_date') . ' >= ' . $db->q($startDate)
                . ' AND ' . $db->qn('end_date') . ' <= ' . $db->q($endDate) . ') OR (' . $db->qn('end_date') . ' >= ' . $db->q($startDate)
                . ' AND ' . $db->qn('start_date') . ' <= '. $db->q($endDate) .'))');

        $query = $db->getQuery(true);

        $query->select($db->qn(array('id', 'name')))
            ->from($db->qn('#__jroombooking_rooms'))
            ->where($db->qn('published') . ' = ' . $db->q(1) . ' AND ' . $db->qn('max_capacity') . ' >= ' . $capacity . ' AND ' . $db->qn('id') . ' NOT IN (' . $subQuery . ')');

        $db->setQuery($query);
        $result = $db->loadObjectList();

        if ($result == 0) return false;
        return $result;
    }
}